"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * @copyright 20109 ChoWonmin <0108257@gmail.com>
 * @license MIT
 * @description Matrix library
 */

var matrix = function matrix(c, r) {
  var value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;


  var row = r;
  var col = c;
  var elements = new Array(col);

  var init = function init() {
    for (var i = 0; i < col; i++) {
      elements[i] = new Array(row);
      for (var j = 0; j < row; j++) {
        elements[i][j] = value;
      }
    }
  };

  var getRow = function getRow() {
    return row;
  };

  var getCol = function getCol() {
    return col;
  };

  var getValue = function getValue(c, r) {
    return elements[c][r];
  };

  var setValue = function setValue(c, r, value) {
    return elements[c][r] = value;
  };

  var each = function each(action) {
    elements.forEach(function (row, col) {
      row.forEach(function (e, i) {
        action(e, col, i);
      });
    });
  };

  var clone = function clone() {
    var res = matrix(col, row);

    res.each(function (e, c, r) {
      res.setValue(c, r, elements[c][r]);
    });
    return res;
  };

  var add = function add(mat) {
    if (row === mat.getRow() && col === mat.getCol()) {
      var res = clone();
      res.each(function (e, c, r) {
        res.setValue(c, r, e + mat.getValue(c, r));
      });

      return res;
    }
    return null;
  };

  var multiScala = function multiScala(k) {
    var res = clone();

    res.each(function (e, c, r) {
      setValue(c, r, e * k);
    });

    return res;
  };

  var multi = function multi(mat) {
    if (row === mat.getCol()) {
      var res = matrix(col, mat.getRow());

      res.each(function (e, c, r) {
        var val = 0;

        for (var i = 0; i < row; i++) {
          val += getValue(c, i) * mat.getValue(i, r);
        }

        res.setValue(c, r, val);
      });

      return res;
    }
    return null;
  };

  var toArray2D = function toArray2D() {
    return elements;
  };

  var toArray = function toArray() {
    var res = [];

    elements.forEach(function (row) {
      row.forEach(function (e) {
        return res.push(e);
      });
    });

    return res;
  };

  init();

  return {
    getRow: getRow,
    getCol: getCol,
    getValue: getValue,
    setValue: setValue,
    each: each,
    clone: clone,
    add: add,
    multiScala: multiScala,
    multi: multi,
    toArray2D: toArray2D,
    toArray: toArray
  };
};

exports.default = matrix;