'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * datastructures-js/priority-queue
 * @copyright 2018 Eyas Ranjous <eyas.ranjous@gmail.com>
 * @license MIT
 */
var priorityQueue = function priorityQueue() {
  var elements = [];
  var offset = 0;
  var priorities = {};

  /**
   * @returns {number}
   */
  var length = function length() {
    return elements.length - offset;
  };

  /**
   * @returns {boolean}
   */
  var isEmpty = function isEmpty() {
    return length() === 0;
  };

  /**
   * @returns {array}
   */
  var toArray = function toArray() {
    var arr = [];
    if (isEmpty()) {
      return arr;
    }
    var pKeys = Object.keys(priorities);
    for (var i = offset; i < pKeys.length; i += 1) {
      arr.push(elements[priorities[pKeys[i]]]);
    }
    return arr;
  };

  /**
   * clears the queue
   */
  var clear = function clear() {
    elements = [];
    offset = 0;
    priorities = {};
  };

  /**
   * @returns {object}
   */
  var front = function front() {
    if (!isEmpty()) {
      var pKeys = Object.keys(priorities);
      var firstIndex = priorities[pKeys[offset]];
      return elements[firstIndex];
    }
    return null;
  };

  /**
   * @returns {object}
   */
  var back = function back() {
    if (!isEmpty()) {
      var pKeys = Object.keys(priorities);
      var lastIndex = priorities[pKeys[pKeys.length - 1]];
      return elements[lastIndex];
    }
    return null;
  };

  /**
   * @param {object} element
   * @param {number} priority
   * @throws {Error}
   */
  var enqueue = function enqueue(el, priority) {
    if (Number.isNaN(+priority) || priority < 1) {
      throw new Error('priority should be a positive number');
    }
    elements.push(el);
    priorities[priority] = elements.length - 1;
  };

  /**
   * only remove dequeued elements when reaching half size
   * to improve performance for high frequency data
   * @returns {object}
   */
  var dequeue = function dequeue() {
    if (!isEmpty()) {
      var first = front();
      offset += 1;
      if (offset * 2 >= elements.length) {
        var pKeys = Object.keys(priorities);
        var els = [];
        var prs = {};
        for (var i = offset; i < elements.length; i += 1) {
          els.push(elements[priorities[pKeys[i]]]);
          prs[priorities[i]] = els.length - 1;
        }
        priorities = prs;
        elements = els;
        offset = 0;
      }
      return first;
    }
    return null;
  };

  // priority queue API
  return {
    length: length,
    isEmpty: isEmpty,
    toArray: toArray,
    clear: clear,
    front: front,
    back: back,
    enqueue: enqueue,
    dequeue: dequeue
  };
};

exports.default = priorityQueue;