"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * datastructures-js/queue
 * @copyright 2018 Eyas Ranjous <https://github.com/eyas-ranjous>
 * @license MIT
 */
var queue = function queue() {
  var elements = [];
  var offset = 0;

  var length = function length() {
    return elements.length - offset;
  };

  var isEmpty = function isEmpty() {
    return length() === 0;
  };

  var toArray = function toArray() {
    return elements.slice(offset);
  };

  var clear = function clear() {
    offset = 0;
    elements = [];
  };

  var front = function front() {
    if (!isEmpty()) {
      return elements[offset];
    }
    return null;
  };

  var back = function back() {
    if (!isEmpty()) {
      return elements[elements.length - 1];
    }
    return null;
  };

  var enqueue = function enqueue(el) {
    return elements.push(el);
  };

  // only remove dequeued elements when reaching half size
  // to improve performance for high frequency data
  var dequeue = function dequeue() {
    if (!isEmpty()) {
      var first = front();
      offset += 1;
      if (offset * 2 >= elements.length) {
        elements = elements.slice(offset);
        offset = 0;
      }
      return first;
    }
    return null;
  };

  return {
    length: length,
    isEmpty: isEmpty,
    toArray: toArray,
    clear: clear,
    front: front,
    back: back,
    enqueue: enqueue,
    dequeue: dequeue
  };
};

exports.default = queue;