"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * datastructures-js/set
 * @class
 * @copyright 2018 Eyas Ranjous <eyas.ranjous@gmail.com>
 * @license MIT
 */
var set = function set() {
  var elements = {};
  var count = 0;

  /**
   * @returns {number}
   */
  var size = function size() {
    return count;
  };

  /**
   * @returns {boolean}
   */
  var isEmpty = function isEmpty() {
    return count === 0;
  };

  /**
   * @param {(string|number|boolean)} el
   * @returns {boolean}
   */
  var contains = function contains(el) {
    return elements[el] === true;
  };

  /**
   * @param {(string|number|boolean)} el
   */
  var add = function add(el) {
    if (!elements[el]) {
      elements[el] = true;
      count += 1;
    }
  };

  /**
   * @param {(string|number|boolean)} el
   */
  var remove = function remove(el) {
    if (elements[el]) {
      elements[el] = false;
      count -= 1;
    }
  };

  /**
   * @returns {array}
   */
  var toArray = function toArray() {
    var els = [];
    Object.keys(elements).forEach(function (el) {
      if (contains(el)) {
        els.push(Number.isNaN(+el) ? el : +el);
      }
    });
    return els;
  };

  /**
   * @param {object} s - set to union with
   * @returns {object} - resulting set
   */
  var union = function union(s) {
    var unionSet = set();
    Object.keys(elements).forEach(function (el) {
      if (contains(el)) {
        unionSet.add(el);
      }
    });
    s.toArray().forEach(unionSet.add);
    return unionSet;
  };

  /**
   * @param {object} s - set to intersect with
   * @returns {object} - resulting set
   */
  var intersect = function intersect(s) {
    var intersectSet = set();
    Object.keys(elements).forEach(function (el) {
      if (contains(el) && s.contains(el)) {
        intersectSet.add(el);
      }
    });
    return intersectSet;
  };

  /**
   * @param {object} s - set to diff from
   * @returns {object} - resulting set
   */
  var diff = function diff(s) {
    var diffSet = set();
    Object.keys(elements).forEach(function (el) {
      if (contains(el) && !s.contains(el)) {
        diffSet.add(el);
      }
    });
    return diffSet;
  };

  /**
   * @param {object} s -  set to check if it contains the set
   * @returns {boolean}
   */
  var isSubsetOf = function isSubsetOf(s) {
    var result = true;
    Object.keys(elements).forEach(function (el) {
      if (contains(el) && !s.contains(el)) {
        result = false;
      }
    });
    return result;
  };

  /**
   * @param {object} s -  set to check if it is a child of the set
   * @returns {boolean}
   */
  var isSupersetOf = function isSupersetOf(s) {
    var result = true;
    s.toArray().forEach(function (el) {
      if (!contains(el)) {
        result = false;
      }
    });
    return result;
  };

  /**
   * clears the set
   */
  var clear = function clear() {
    elements = {};
    count = 0;
  };

  // set API
  return {
    size: size,
    isEmpty: isEmpty,
    contains: contains,
    add: add,
    remove: remove,
    toArray: toArray,
    union: union,
    intersect: intersect,
    diff: diff,
    isSubsetOf: isSubsetOf,
    isSupersetOf: isSupersetOf,
    clear: clear
  };
};

exports.default = set;