'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * @copyright 2019 ChoWonmin <0108257@gmail.com>
 * @license MIT
 * @description priority queue visualization library
 * @param {Object} [renderer] d3 render object
 * @param {Object} [datastructure] jagu datastructure
 */

var priorityQueueVis = function priorityQueueVis(renderer, datastructure) {

  var root = renderer;
  var backgroundG = root.append('g');
  var foregroundG = root.append('g');
  var activeG = root.append('g');

  var width = root.node().getBoundingClientRect().width;

  var height = root.node().getBoundingClientRect().height;

  var data = datastructure;

  var lock = false;

  var margin = 30;

  var boxProps = {
    width: 140,
    height: 30,
    color: '#ff6f74',
    fontColor: '#dedede',
    padding: 6
  };

  var duration = 1000;

  var drawBox = function drawBox(x, y, text) {
    var rect = foregroundG.append('rect').attr('x', x).attr('y', y).attr('width', boxProps.width - 2 * boxProps.padding).attr('height', boxProps.height).attr('fill', '#eee17f').attr('opacity', 1);

    var t = foregroundG.append('text').attr('x', x + boxProps.width / 2 - 4).attr('y', y + boxProps.height / 2).attr('fill', '#4b4b4b').attr('alignment-baseline', 'middle').attr('text-anchor', 'middle').text(text);

    return {
      rect: rect,
      text: t
    };
  };

  var scroll = function scroll(target) {
    var startY = 0;
    var diffY = 0;
    var moveY = 0;

    target.call(d3.drag().on('start', function () {
      startY = d3.event.y;
    }).on('drag', function () {
      diffY = d3.event.y - startY;

      if (moveY + diffY <= 0) {
        diffY = 0;
      } else {
        target.selectAll('*').attr('transform', 'translate(0, ' + (moveY + diffY) + ')');
      }
    }).on('end', function () {
      startY = 0;
      moveY += diffY;
      diffY = 0;
    }));
  };

  var draw = function draw() {
    clear();

    backgroundG.append('rect').attr('x', margin).attr('y', 0).attr('width', boxProps.width).attr('height', height).attr('fill', '#adefde');

    data.toArray().forEach(function (e, i) {
      var box = drawBox(margin + boxProps.padding, height - (boxProps.height + boxProps.padding) * (i + 1), e);

      box.rect.transition().duration(duration).attr('fill', boxProps.color);

      box.text.transition().duration(duration).attr('fill', boxProps.fontColor);
    });

    scroll(foregroundG);
  };

  var enqueue = function enqueue(ele, priority) {
    data.enqueue(ele, priority);
    draw();
  };

  var dequeue = function dequeue() {
    data.dequeue();

    var front = {
      rect: foregroundG.selectAll('rect').filter(function (d, i) {
        return i === 0;
      }),
      text: foregroundG.selectAll('text').filter(function (d, i) {
        return i === 0;
      })
    };

    front.rect.transition().duration(duration).attr('y', height);

    front.text.transition().duration(duration).attr('y', height).on('end', function () {
      front.rect.remove();
      front.text.remove();

      foregroundG.selectAll('rect').transition().delay(duration).duration(duration).attr('y', function (e, i) {
        return height - (boxProps.height + boxProps.padding) * (i + 1);
      });

      foregroundG.selectAll('text').transition().delay(duration).duration(duration).attr('y', function (e, i) {
        return height + boxProps.height / 2 - (boxProps.height + boxProps.padding) * (i + 1);
      });
    });
  };

  var clear = function clear() {
    backgroundG.selectAll('*').remove();
    foregroundG.selectAll('*').remove();
    activeG.selectAll('*').remove();
  };

  return {
    draw: draw,
    enqueue: enqueue,
    dequeue: dequeue,
    clear: clear
  };
};

exports.default = priorityQueueVis;