'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * @copyright 2019 ChoWonmin <0108257@gmail.com>
 * @license MIT
 * @description stack visualization library
 * @param {Object} [renderer] d3 render object
 * @param {Object} [datastructure] jagu datastructure
 */

var stackVis = function stackVis(renderer, datastructure) {

  var root = renderer;
  var backgroundG = root.append('g');
  var foregroundG = root.append('g');
  var activeG = root.append('g');

  var width = root.node().getBoundingClientRect().width;
  var height = root.node().getBoundingClientRect().height;

  var data = datastructure;

  /**
   * animation synchronize check
   * @property {boolean}
   */
  var lock = false;

  var margin = 30;

  var boxProps = {
    width: 140,
    height: 30,
    color: '#ff6f74',
    fontColor: '#dedede',
    padding: 6
  };

  var duration = 400;

  var drawBox = function drawBox(x, y, text) {
    var rect = foregroundG.append('rect').attr('x', x).attr('y', y).attr('width', boxProps.width - 2 * boxProps.padding).attr('height', boxProps.height).attr('fill', boxProps.color).attr('opacity', 1);

    var t = foregroundG.append('text').attr('x', x + boxProps.width / 2 - 4).attr('y', y + boxProps.height / 2).attr('fill', boxProps.fontColor).attr('alignment-baseline', 'middle').attr('text-anchor', 'middle').text(text);

    return {
      rect: rect,
      text: t
    };
  };

  var scroll = function scroll(target) {
    var startY = 0;
    var diffY = 0;
    var moveY = 0;

    target.call(d3.drag().on('start', function () {
      startY = d3.event.y;
    }).on('drag', function () {
      diffY = d3.event.y - startY;

      if (moveY + diffY <= 0) {
        diffY = 0;
      } else {
        target.selectAll('*').attr('transform', 'translate(0, ' + (moveY + diffY) + ')');
      }
    }).on('end', function () {
      startY = 0;
      moveY += diffY;
      diffY = 0;
    }));
  };

  var draw = function draw() {
    clear();

    backgroundG.append('rect').attr('x', margin).attr('y', 0).attr('width', boxProps.width).attr('height', height).attr('fill', '#adefde');

    data.toArray().forEach(function (e, i) {
      drawBox(margin + boxProps.padding, height - (boxProps.height + boxProps.padding) * (i + 1), e);
    });

    scroll(foregroundG);
  };

  var push = function push(ele) {

    if (!lock) {
      lock = true;
      data.push(ele);

      var box = drawBox(margin + boxProps.padding, 0, ele);

      box.rect.transition().duration(duration).attr('y', height - (boxProps.height + boxProps.padding) * data.toArray().length);

      box.text.transition().duration(duration).attr('y', height + boxProps.height / 2 - (boxProps.height + boxProps.padding) * data.toArray().length).on('end', function () {
        lock = false;
      });
    }
  };

  var pop = function pop() {

    if (!lock) {
      lock = true;
      data.pop();

      var top = {
        rect: foregroundG.selectAll('rect').filter(function (d, i, list) {
          return i === list.length - 1;
        }),
        text: foregroundG.selectAll('text').filter(function (d, i, list) {
          return i === list.length - 1;
        })
      };

      top.rect.transition().duration(duration).attr('y', -boxProps.height);

      top.text.transition().duration(duration).attr('y', -boxProps.height).on('end', function () {
        top.rect.remove();
        top.text.remove();
        lock = false;
      });
    }
  };

  var clear = function clear() {
    backgroundG.selectAll('*').remove();
    foregroundG.selectAll('*').remove();
    activeG.selectAll('*').remove();
  };

  return {
    draw: draw,
    push: push,
    pop: pop,
    clear: clear
  };
};

exports.default = stackVis;