'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * @copyright 2019 ChoWonmin <0108257@gmail.com>
 * @license MIT
 * @description datastructure visualization library
 * @param {Object} [renderer] d3 render object
 * @param {Object} [properties] Box properties :: width, height, color, fontColor
 */

var box = function box(renderer) {
  var properties = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};


  var root = renderer;

  var box = {
    rect: undefined,
    text: undefined
  };

  var props = {
    width: properties.width || 140,
    height: properties.height || 30,
    color: properties.color || '#ff6f74',
    fontColor: properties.fontColor || '#dedede'
  };

  var draw = function draw(x, y, text) {
    var properties = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

    box.rect = root.append('rect').attr('x', x).attr('y', y).attr('width', props.width - 2 * props.padding).attr('height', props.height).attr('fill', props.color).attr('opacity', 1);

    box.text = root.append('text').attr('x', x + props.width / 2).attr('y', y + props.height / 2).attr('fill', props.fontColor).attr('alignment-baseline', 'middle').text(text);
  };

  var move = function move(x, y) {
    var transition = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var transitionProps = {
      duration: transition || 700
    };

    box.rect.transition().duration(transitionProps.duration).attr('x', x).attr('y', y);

    box.text.transition().duration(transitionProps.duration).attr('x', x).attr('y', y + props.height / 2).on('end', function () {
      draw();
    });
  };

  var clear = function clear() {
    box.rect.remove();
    box.text.remove();
  };

  return {
    box: box,
    draw: draw,
    move: move,
    clear: clear
  };
};

exports.default = box;